@@ -510,6 +510,7 @@ initialise_error(E) :-
 
 initialise_prolog :-
     '$clean_history',
+    apple_setup_app,
     '$run_initialization',
     '$load_system_init_file',
     set_toplevel,
@@ -534,6 +535,32 @@ initialise_prolog :-
         )
     ).
 
+:- if(current_prolog_flag(apple,true)).
+apple_set_working_directory :-
+    (   expand_file_name('~', [Dir]),
+	exists_directory(Dir)
+    ->  working_directory(_, Dir)
+    ;   true
+    ).
+
+apple_set_locale :-
+    (   getenv('LC_CTYPE', 'UTF-8'),
+	apple_current_locale_identifier(LocaleID),
+	atom_concat(LocaleID, '.UTF-8', Locale),
+	catch(setlocale(ctype, _Old, Locale), _, fail)
+    ->  setenv('LANG', Locale),
+        unsetenv('LC_CTYPE')
+    ;   true
+    ).
+
+apple_setup_app :-
+    current_prolog_flag(apple, true),
+    current_prolog_flag(console_menu, true),	% SWI-Prolog.app on MacOS
+    apple_set_working_directory,
+    apple_set_locale.
+:- endif.
+apple_setup_app.
+
 opt_attach_packs :-
     current_prolog_flag(packs, true),
     !,
@@ -871,6 +898,13 @@ read_expanded_query(BreakLev, ExpandedQuery, ExpandedBindings) :-
 %   !-based history is enabled. The second is   used  if we have command
 %   line editing.
 
+:- if(current_prolog_flag(emscripten, true)).
+read_query(_Prompt, Goal, Bindings) :-
+    '$can_yield',
+    !,
+    js_yield(goal, GoalString),
+    term_string(Goal, GoalString, [variable_names(Bindings)]).
+:- endif.
 read_query(Prompt, Goal, Bindings) :-
     current_prolog_flag(history, N),
     integer(N), N > 0,
@@ -1598,6 +1632,13 @@ self_bounded(binding([Name], Value, [])) :-
 %
 %   Read the continuation entered by the user.
 
+:- if(current_prolog_flag(emscripten, true)).
+get_respons(Action, _Chp) :-
+    '$can_yield',
+    !,
+    js_yield(more, ActionS),
+    atom_string(Action, ActionS).
+:- endif.
 get_respons(Action, Chp) :-
     repeat,
         flush_output(user_output),
