@@ -83,7 +83,9 @@
 	      prolog_close_source/1,
 	      prolog_read_source_term/4
 	    ]).
+:- if(exists_source(library(shlib))).
 :- autoload(library(shlib),[current_foreign_library/2]).
+:- endif.
 :- autoload(library(solution_sequences),[distinct/2,limit/2]).
 
 :- if(exists_source(library(pldoc))).
@@ -354,10 +356,16 @@ xref_setup(Src, In, Options, state(In, Dialect, Xref, [SRef|HRefs])) :-
     set_xref(Xref),
     (   verbose(Src)
     ->  HRefs = []
-    ;   asserta(user:thread_message_hook(_,_,_), Ref),
+    ;   asserta((user:thread_message_hook(_,Level,_) :-
+                     hide_message(Level)),
+                Ref),
         HRefs = [Ref]
     ).
 
+hide_message(warning).
+hide_message(error).
+hide_message(informational).
+
 assert_option(_, Var) :-
     var(Var),
     !,
