@@ -447,7 +447,7 @@ getString(IOSTREAM *fd, size_t *length)
   size_t len = (size_t)getInt64(fd);
   size_t i;
 
-  if ( !length && len > MAXPATHLEN )
+  if ( !length && len > PATH_MAX )
     return NULL;
   if ( length && len > globalStackLimit() )
     return NULL;
@@ -1710,7 +1710,7 @@ loadImport(DECL_LD wic_state *state, int skip)
 
 static atom_t
 qlfFixSourcePath(wic_state *state, const char *raw)
-{ char buf[MAXPATHLEN];
+{ char buf[PATH_MAX];
   char *canonical;
 
   if ( state->load_state->has_moved &&
@@ -1719,14 +1719,14 @@ qlfFixSourcePath(wic_state *state, const char *raw)
     size_t lensave = strlen(state->load_state->save_dir);
     const char *tail = &raw[lensave];
 
-    if ( strlen(state->load_state->load_dir)+1+strlen(tail)+1 > MAXPATHLEN )
+    if ( strlen(state->load_state->load_dir)+1+strlen(tail)+1 > PATH_MAX )
       fatalError("Path name too long: %s", raw);
 
     strcpy(buf, state->load_state->load_dir);
     s = &buf[strlen(buf)];
     strcpy(s, tail);
   } else
-  { if ( strlen(raw)+1 > MAXPATHLEN )
+  { if ( strlen(raw)+1 > PATH_MAX )
     { fatalError("Path name too long: %s", raw);
       return NULL_ATOM;
     }
@@ -3104,7 +3104,40 @@ qlfSourceInfo(DECL_LD wic_state *state, size_t offset, term_t list)
 }
 
 
-#define qlfInfo(file, cversion, minload, fversion, csig, fsig, files0) LDFUNC(qlfInfo, file, cversion, minload, fversion, csig, fsig, files0)
+static int
+open_qlf_file(const char *file, IOSTREAM **sp)
+{ int sl;
+
+  if ( (sl=file_name_is_iri(file)) )
+  { IOSTREAM *s;
+
+    if ( !iri_hook(file, IRI_OPEN, ATOM_read, 0, &s) )
+      return FALSE;
+    s->encoding = ENC_OCTET;
+    clear(s, SIO_TEXT);
+
+    *sp = s;
+
+    return TRUE;
+  } else
+  { if ( (*sp = Sopen_file(file, "rbr")) )
+    { return TRUE;
+    } else
+    { GET_LD
+      term_t f = PL_new_term_ref();
+
+      PL_put_atom_chars(f, file);
+      return PL_error(NULL, 0, OsError(), ERR_FILE_OPERATION,
+		      ATOM_open, ATOM_source_sink, f);
+    }
+  }
+}
+
+
+
+#define qlfInfo(file, cversion, minload, fversion, csig, fsig, files0) \
+	LDFUNC(qlfInfo, file, cversion, minload, fversion, csig, fsig, files0)
+
 static word
 qlfInfo(DECL_LD const char *file,
 	term_t cversion, term_t minload, term_t fversion,
@@ -3115,19 +3148,13 @@ qlfInfo(DECL_LD const char *file,
   int nqlf, i;
   size_t *qlfstart = NULL;
   word rval = FALSE;
-  term_t files = PL_copy_term_ref(files0);
   wic_state state;
 
   memset(&state, 0, sizeof(state));
   state.wicFile = (char*)file;
 
-  if ( !(s = Sopen_file(file, "rbr")) )
-  { term_t f = PL_new_term_ref();
-
-    PL_put_atom_chars(f, file);
-    return PL_error(NULL, 0, OsError(), ERR_FILE_OPERATION,
-		    ATOM_open, ATOM_source_sink, f);
-  }
+  if ( !open_qlf_file(file, &s) )
+    return FALSE;
   state.wicFd = s;
 
   if ( cversion )
@@ -3154,36 +3181,39 @@ qlfInfo(DECL_LD const char *file,
   if ( !pushPathTranslation(&state, file, 0) )
     goto out;
 
-  if ( Sseek(s, -4, SIO_SEEK_END) < 0 )	/* 4 bytes of PutInt32() */
-  { qlfError(&state, "seek to index failed: %s", OsError());
-    goto out;
-  }
-  if ( (nqlf = getInt32(s)) < 0 )
-  { qlfError(&state, "invalid number of files (%d)", nqlf);
-    goto out;
-  }
-  if ( Sseek(s, -4 * (nqlf+1), SIO_SEEK_END) < 0 )
-  { qlfError(&state, "seek to files failed: %s", OsError());
-    goto out;
-  }
-
-  DEBUG(MSG_QLF_SECTION, Sdprintf("Found %d sources at", nqlf));
-  if ( !(qlfstart = malloc(sizeof(size_t)*nqlf)) )
-  { PL_no_memory();
-    goto out;
-  }
-  for(i=0; i<nqlf; i++)
-  { qlfstart[i] = (size_t)getInt32(s);
-    DEBUG(MSG_QLF_SECTION, Sdprintf(" %ld", qlfstart[i]));
-  }
-  DEBUG(MSG_QLF_SECTION, Sdprintf("\n"));
-
-  for(i=0; i<nqlf; i++)
-  { if ( !qlfSourceInfo(&state, qlfstart[i], files) )
+  if ( files0 )
+  { term_t files = PL_copy_term_ref(files0);
+    if ( Sseek(s, -4, SIO_SEEK_END) < 0 )	/* 4 bytes of PutInt32() */
+    { qlfError(&state, "seek to index failed: %s", OsError());
       goto out;
-  }
+    }
+    if ( (nqlf = getInt32(s)) < 0 )
+    { qlfError(&state, "invalid number of files (%d)", nqlf);
+      goto out;
+    }
+    if ( Sseek(s, -4 * (nqlf+1), SIO_SEEK_END) < 0 )
+    { qlfError(&state, "seek to files failed: %s", OsError());
+      goto out;
+    }
 
-  rval = PL_unify_nil(files);
+    DEBUG(MSG_QLF_SECTION, Sdprintf("Found %d sources at", nqlf));
+    if ( !(qlfstart = malloc(sizeof(size_t)*nqlf)) )
+    { PL_no_memory();
+      goto out;
+    }
+    for(i=0; i<nqlf; i++)
+    { qlfstart[i] = (size_t)getInt32(s);
+      DEBUG(MSG_QLF_SECTION, Sdprintf(" %ld", qlfstart[i]));
+    }
+    DEBUG(MSG_QLF_SECTION, Sdprintf("\n"));
+
+    for(i=0; i<nqlf; i++)
+    { if ( !qlfSourceInfo(&state, qlfstart[i], files) )
+	goto out;
+    }
+
+    rval = PL_unify_nil(files);
+  }
 
 out:
   popPathTranslation(&state);
@@ -3200,6 +3230,9 @@ out:
 		-CurrentVersion, -MinLOadVersion, -FileVersion,
 		-CurrentSignature, -FileSignature,
 		-Files)
+    '$qlf_info'(+File,
+		-CurrentVersion, -MinLOadVersion, -FileVersion,
+		-CurrentSignature, -FileSignature)
 
 Provide information about a QLF file.
 
@@ -3210,6 +3243,18 @@ Provide information about a QLF file.
 @arg Files is a list of atoms representing the files used to create the QLF
 */
 
+static
+PRED_IMPL("$qlf_info", 6, qlf_info, 0)
+{ PRED_LD
+  char *name;
+
+  if ( !PL_get_file_name(A1, &name, PL_FILE_ABSOLUTE) )
+    fail;
+
+  return qlfInfo(name, A2, A3, A4, A5, A6, 0);
+}
+
+
 static
 PRED_IMPL("$qlf_info", 7, qlf_info, 0)
 { PRED_LD
@@ -3218,7 +3263,7 @@ PRED_IMPL("$qlf_info", 7, qlf_info, 0)
   if ( !PL_get_file_name(A1, &name, PL_FILE_ABSOLUTE) )
     fail;
 
-  return qlfInfo(name, A2, A3, A4, A5, A6, A8);
+  return qlfInfo(name, A2, A3, A4, A5, A6, A7);
 }
 
 
@@ -3242,7 +3287,7 @@ static wic_state *
 qlfOpen(term_t file)
 { char *name;
   char *absname;
-  char tmp[MAXPATHLEN];
+  char tmp[PATH_MAX];
   IOSTREAM *out;
   wic_state *state;
 
@@ -3325,12 +3370,12 @@ pushPathTranslation(wic_state *state, const char *absloadname, int flags)
     return qlfError(state, "bad string");
 
   if ( absloadname && !streq(absloadname, abssavename) )
-  { char load[MAXPATHLEN];
-    char save[MAXPATHLEN];
+  { char load[PATH_MAX];
+    char save[PATH_MAX];
     char *l, *s, *le, *se;
 
-    if ( ( strlen(abssavename)+1 > MAXPATHLEN ||
-	   strlen(absloadname)+1 > MAXPATHLEN
+    if ( ( strlen(abssavename)+1 > PATH_MAX ||
+	   strlen(absloadname)+1 > PATH_MAX
 	 ) )
       return PL_representation_error("max_path_length");
 
@@ -3434,7 +3479,7 @@ qlfLoad(DECL_LD wic_state *state, Module *module)
 { IOSTREAM *fd = state->wicFd;
   bool rval;
   const char *absloadname;
-  char tmp[MAXPATHLEN];
+  char tmp[PATH_MAX];
   atom_t file;
 
   if ( (file = fileNameStream(fd)) )
@@ -3799,7 +3844,7 @@ PRED_IMPL("$qlf_load", 2, qlf_load, PL_FA_TRANSPARENT)
 Write a header for a QLF-stream
 */
 
-static const opt_spec open_wic_options[] =
+static const PL_option_t open_wic_options[] =
 { { ATOM_obfuscate,	    OPT_BOOL },
   { NULL_ATOM,		    0 }
 };
@@ -3813,7 +3858,7 @@ PRED_IMPL("$open_wic", 2, open_wic, 0)
 
   assert(V_LABEL > I_HIGHEST);
 
-  if ( !scan_options(A2, 0, ATOM_state_option, open_wic_options,
+  if ( !PL_scan_options(A2, 0, "state_option", open_wic_options,
 		     &obfuscate) )
     fail;
 
@@ -4085,7 +4130,7 @@ directiveClause(term_t directive, term_t clause, const char *functor)
 static bool
 compileFile(wic_state *state, const char *file)
 { GET_LD
-  char tmp[MAXPATHLEN];
+  char tmp[PATH_MAX];
   char *path;
   term_t f = PL_new_term_ref();
   SourceFile sf;
@@ -4251,6 +4296,7 @@ wicPutStringW(const pl_wchar_t *w, size_t len, IOSTREAM *fd)
 		 *******************************/
 
 BeginPredDefs(wic)
+  PRED_DEF("$qlf_info",		    6, qlf_info,	     0)
   PRED_DEF("$qlf_info",		    7, qlf_info,	     0)
   PRED_DEF("$qlf_sources",	    2, qlf_sources,	     0)
   PRED_DEF("$qlf_load",		    2, qlf_load,	     PL_FA_TRANSPARENT)
